/*
 * Copyright 1996 1995 by Open Software Foundation, Inc. 1997 1996 1995 1994 1993 1992 1991  
 *              All Rights Reserved 
 *  
 * Permission to use, copy, modify, and distribute this software and 
 * its documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appears in all copies and 
 * that both the copyright notice and this permission notice appear in 
 * supporting documentation. 
 *  
 * OSF DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE 
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE. 
 *  
 * IN NO EVENT SHALL OSF BE LIABLE FOR ANY SPECIAL, INDIRECT, OR 
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM 
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, 
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */
/*
 * MkLinux
 */

#ifndef	_MACHINE_SPL_H_
#define	_MACHINE_SPL_H_

/*
 *	This file defines the interrupt priority levels used by
 *	machine-dependent code.
 *
 *      The powerpc has only primitive interrupt masking (all/none),
 *      spl behaviour is done in software
 */

#include <mach_kdb.h>

/*
 * These are the bits in the eirr register that are assigned for various 
 * devices
 */

#define SPL_CLOCK_BIT		0
#define SPL_POWER_BIT		1
#define SPL_VM_BIT		4
#define SPL_CIO_BIT		8
#define SPL_HPIB_BIT		9
#define SPL_BIO_BIT		9
#define SPL_IMP_BIT		9
#define SPL_TTY_BIT		16
#define SPL_NET_BIT		24

/*
 * The powerpc only has a single level of hardware interrupt. There is
 * no processor support for disabling certain interrupts whilst allowing
 * others - this must be done in software.
 * 
 * The mechanism chosen is fairly standard - a primary interrupt fielder
 * is used which detects the source of the incoming interrupt and thus its
 * priority. This routine will either allow the interrupt to pass to
 * its handler if it is above the current priority, or queue it if it is
 * not. A word of 32 bits is used for queue flags, thus determining the
 * number of interrupt levels available.
 */


/*
 * Convert interrupt levels into machine-independent SPLs as follows:
 *
 *                        1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3
 *  0 1 2 3 4 5 6 7  8  9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |c|p| | |v| | | |b i| | | | | | | |t| | | | | | | |n| | |s| | | | |
 * |l|w| | |m| | | |i m| | | | | | | |t| | | | | | | |e| | |c| | | | |
 * |k|r| | | | | | |o p| | | | | | | |y| | | | | | | |t| | |l| | | | |
 * | | | | | | | | |   | | | | | | | | | | | | | | | | | | |k| | | | |
 * +-+-+-+-+-+-+-+-+---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 * Notes:
 *	- software prohibits more than one machine-dependent SPL per bit on
 *	  a given architecture (e.g. ppc).  In cases where there
 *	  are multiple equivalent devices which interrupt at the same level
 *	  (e.g. ASP RS232 #1 and #2), the interrupt table insertion routine
 *	  will always add in the unit number (arg0) to offset the entry.
 *	- hard clock must be the first bit (i.e. 0x80000000).
 *	- SPL7 is any non-zero value (since the PSW I-bit is off).
 *	- SPLIMP serves two purposes: blocks network interfaces and blocks
 *	  memory allocation via malloc.  In theory, SPLLAN would be high
 *TODONMGS	  enough.  However, on the hp700, the SCSI driver uses malloc at
 *TODONMGS	  interrupt time requiring SPLIMP >= SPLBIO.  On the 800, we are
 *	  still using HP-UX drivers which make the assumption that
 *	  SPLIMP >= SPLCIO.  New drivers would address both problems.
 */
/* TODO NMGS THESE SPL NUMBERS ARE WRONG!!!!! DANGER !!! */
 /* Note also : if any new SPL's are introduced, please add to debugging list*/
#define SPLOFF		0       /* all interrupts disabled TODO NMGS  */
#define	SPLHIGH	        2	/* TODO NMGS any non-zero, non-INTPRI value */
#define SPLSCHED	SPLHIGH
#define	SPLCLOCK	0	/* hard clock */
#define	SPLPOWER	1	/* power failure (unused) */
#define	SPLVM		4	/* TLB shootdown (unused) */
#define	SPLBIO		8	/* block I/O */
#define	SPLIMP		8	/* network & malloc */
#define	SPLTTY		16	/* TTY */
#define	SPLNET		24	/* soft net */
#define	SPLSCLK		27	/* soft clock */
#define	SPLLO		32	/* no interrupts masked */

#define SPL_CMP_GT(a, b)        ((unsigned)(a) >  (unsigned)(b))
#define SPL_CMP_LT(a, b)        ((unsigned)(a) <  (unsigned)(b))
#define SPL_CMP_GE(a, b)        ((unsigned)(a) >= (unsigned)(b))
#define SPL_CMP_LE(a, b)        ((unsigned)(a) <= (unsigned)(b))

#ifndef ASSEMBLER
/*
 * This is a generic interrupt switch table.  It may be used by various
 * interrupt systems.  For each interrupt, it holds a handler and an
 * TODO NMGS interrupt mask (selected from SPL* or, more generally, INTPRI*).
 *
 * So that these tables can be easily found, please prefix them with
 * the label "itab_" (e.g. "itab_proc").
 */

typedef unsigned spl_t;

#include <mach_debug.h>

#if 0
#if MACH_DEBUG
struct {
	spl_t level;
	char* name;
} spl_names[] = {
	{ SPLHIGH,	"SPLHIGH" },
	{ SPLCLOCK,	"SPLCLOCK" },
	{ SPLPOWER,     "SPLPOWER" },
	{ SPLVM,	"SPLVM" },
	{ SPLBIO,	"SPLBIO" },
	{ SPLIMP,	"SPLIMP" },
	{ SPLTTY,	"SPLTTY" },
	{ SPLNET,	"SPLNET" },
	{ SPLSCLK,	"SPLSCLK" },
	{ SPLLO,	"SPLLO" },
};

#define SPL_NAMES_COUNT (sizeof (spl_names)/sizeof(spl_names[0]))

#endif /* DEBUG */
#endif /* 0 */
struct intrtab {
	void (*handler)(int); /* ptr to routine to call */
	unsigned int intpri;	/* INTPRI (SPL) with which to call it */
	int arg;		/* 1 arguments to handler: arg0 is unit */
};

extern spl_t set_priority_level(spl_t level);

spl_t   sploff(void);
spl_t 	splhigh(void);
spl_t	splsched(void);
spl_t 	splclock(void);
spl_t 	splpower(void);
spl_t 	splvm(void);
spl_t 	splbio(void);
spl_t 	splimp(void);
spl_t 	spltty(void);
spl_t 	splnet(void);
spl_t 	splsclk(void);

void 	spllo(void);
void	splon(spl_t level);
void	splx(spl_t level);

#if	MACH_KDB
extern spl_t db_set_priority_level(spl_t level);
#endif	/* MACH_KDB */

#endif /* ASSEMBLER */

#endif	/* _MACHINE_SPL_H_ */
